@@ -1,13 +1,16 @@
 # Copyright (c) 2007-2008 Infrae. All rights reserved.
-# $Id: Native.py 29363 2008-06-19 15:27:24Z sylvain $
+# $Id: Native.py 29942 2008-08-01 12:58:18Z sylvain $
 
-from pysvn import wc_status_kind
+from pysvn import wc_status_kind, opt_revision_kind, wc_notify_action
 import pysvn
 
-import os, os.path
+from Common import BaseRecipe, checkExistPath, prepareURLs
+
+from sets import Set            # For python 2.3 compatibility
+import os
 import re
 
-def createSVNClient():
+def createSVNClient(recipe):
     """Create a pysvn client, and setup some callback and options.
     """
 
@@ -18,81 +21,45 @@ def createSVNClient():
         print "-------- SECURITY WARNING --------"
         return True, 0, False
 
+    def callback_notify(info):
+        if info['action'] == wc_notify_action.update_completed:
+            path = info['path']
+            recipe._updateRevisionInformation(path, recipe.urls[path], info['revision'])
+
     client = pysvn.Client()
     client.set_interactive(True)
     client.callback_ssl_server_trust_prompt = callback_ssl
-
+    if not (recipe is None):
+        client.callback_notify = callback_notify
     return client
 
-
-def checkExistPath(path):
-    """Check that a path exist.
-    """
-    status = os.path.exists(path)
-    if not status:
-        print "-------- WARNING --------"
-        print "Directory %s have been removed." % os.path.abspath(path)
-        print "Changes might be lost."
-        print "-------- WARNING --------"
-    return status
-
-
-class Recipe:
+class Recipe(BaseRecipe):
     """infrae.subversion recipe.
     """
 
     def __init__(self, buildout, name, options):
-        self.buildout = buildout
-        self.name = name
-        self.options = options
-        
-        options['location'] = self.location = os.path.join(
-            buildout['buildout']['parts-directory'], self.name)
-        self.urls = [l.split()
-                     for l in options['urls'].splitlines()
-                     if l.strip()]
-        self.export = options.get('export')
-        self.newest = (
-            buildout['buildout'].get('offline', 'false') == 'false'
-            and
-            buildout['buildout'].get('newest', 'true') == 'true'
-            )
+        super(Recipe, self).__init__(buildout, name, options)
+        self.client = createSVNClient(self)
+        self._updateAllRevisionInformation()
+        self._exportInformationToOptions()
 
-        self.client = createSVNClient()
-        self.verbose = buildout['buildout'].get('verbosity', 0)
 
-    def update(self):
-        """Update the checkouts.
-
-        Does nothing if buildout is in offline mode.
+    def _updateRevisionInformation(self, link, path, revision=None):
+        """Update revision information on a path.
         """
-        if not self.newest:
-            return self.location
+        if revision is None:
+            info = self.client.info(path)
+            revision = info['revision']
 
-        ignore = self.options.get('ignore_updates', False) or self.export
+        assert (revision.kind == opt_revision_kind.number)
+        super(Recipe, self)._updateRevisionInformation(link, revision.number)
 
-        num_release = re.compile('.*@[0-9]+$')
-        for link, sub_path in self.urls:
-            path = os.path.join(self.location, sub_path)
-            if not checkExistPath(path):
-                if self.verbose:
-                    print "Entry %s deleted, checkout a new version ..." % link
-                self._installPath(link, path)
-                continue
 
-            if ignore:
-                continue
-            
-            if num_release.match(link):
-                if self.verbose:
-                    print "Given num release for %s, skipping." % link
-                continue
+    def _updatePath(self, link, path):
+        """Update a single path.
+        """
+        self.client.update(path)
 
-            if self.verbose:
-                print "Updating %s" % path
-            self.client.update(path)
-            
-        return self.location
 
     def _parseRevisionInUrl(self, url):
         """Parse URL to extract revision number. This is not done by
@@ -102,36 +69,22 @@ class Recipe:
         match = num_release.match(url)
         if match:
             return (match.group(1),
-                    pysvn.Revision(pysvn.opt_revision_kind.number,
+                    pysvn.Revision(opt_revision_kind.number,
                                    int(match.group(2))))
-        return (url, pysvn.Revision(pysvn.opt_revision_kind.head))
+        return (url, pysvn.Revision(opt_revision_kind.head))
 
 
     def _installPath(self, link, path):
         """Checkout a single entry.
         """
-        if self.verbose:
-            print "%s %s to %s" % (self.export and 'Export' or 'Fetch',
-                                   link, path)
-            
-        link, revision = self._parseRevisionInUrl(link)
+        link, wanted_revision = self._parseRevisionInUrl(link)
         if self.export:
-            self.client.export(link, path, revision=revision, recurse=True)
+            method = self.client.export
         else:
-            self.client.checkout(link, path, revision=revision, recurse=True)
+            method = self.client.checkout
+        method(link, path, revision=wanted_revision, recurse=True)
 
 
-    def install(self):
-        """Checkout the checkouts.
-
-        Fails if buildout is running in offline mode.
-        """
-
-        for link, sub_path in self.urls:
-            path = os.path.join(self.location, sub_path)
-            self._installPath(link, path)
-
-        return self.location
 
 def uninstall(name, options):
     r"""
@@ -154,8 +107,8 @@ def uninstall(name, options):
     #     options['buildout'] available so no
     #     'buildout:parts-directory'.
     location = options.get('location', os.path.join('.', 'parts', name))
-    
-    client = createSVNClient()
+    urls = prepareURLs(location, options['urls'])
+    client = createSVNClient(None)
 
     bad_svn_status = [wc_status_kind.modified, 
                       wc_status_kind.missing,
@@ -164,14 +117,19 @@ def uninstall(name, options):
     if not checkExistPath(location):
         return
 
-    for sub_path in os.listdir(location):
-        path = os.path.join(location, sub_path)
+    current_paths = Set([os.path.join(location, s) for s in os.listdir(location)])
+    recipe_paths = Set(urls.keys())
+    added_paths = current_paths.difference(recipe_paths)
+    if added_paths:
+        msg = "New path have been added to the location: %s."
+        raise ValueError(msg, ', '.join(added_paths))
+
+    for path in urls.keys():
         if not checkExistPath(path):
             continue
 
         badfiles = filter(lambda e: e['text_status'] in bad_svn_status, 
                           client.status(path))
-    
         if badfiles:
             raise ValueError("""\
 In '%s':
